/*
 * Copyright (c) 2006 The Trustees of Indiana University and Indiana
 *                    University Research and Technology
 *                    Corporation.  All rights reserved.
 * Copyright (c) 2006 The Technical University of Chemnitz. All 
 *                    rights reserved.
 *
 * Author(s): (alphabetically)
 *            Sven Eckelmann
 *            Torsten Hoefler 
 *            Mirko Reinhard 
 *            Matthias Treydte 
 */

#ifndef ENET_MODULE_H_
#define ENET_MODULE_H_


#include "af_enet.h"


/* activate this to get LOTS of debugging output */
#define ENET_DEBUG 0

/* activate this to get profiling output */
#define ENET_PROFILE 0


/* Module error output function */
#define PR_ERR(fmt, arg...) \
 printk(KERN_ERR "ENET: " fmt "\n", ##arg)


/* rdtsc - Read Time-Stamp Counter (64-bit) into EDX:EAX
 * (cpuid guarantees that previous instructions are completed)
 */
static inline void cpu_timestamp(unsigned long long *x)
{
	__asm__ __volatile__(
	        "xorl %%eax, %%eax\n\t"
	        "cpuid\n\t"
	        "rdtsc\n\t"
	        "movl %0, %%ebx\n\t"
	        "movl %%eax, (%%ebx)\n\t"
	        "movl %%edx, 4(%%ebx)\n\t"
	        "xorl %%eax, %%eax\n\t"
	        "cpuid"
        :
        : "m"(x)
			        : "edx", "eax", "ebx", "ecx"
			);
}


#if ENET_DEBUG


/* declare a variable for debugging */
#define ENET_DEBUG_DECLARE(x) x

/* Module debugging output function */
#define PR_DEBUG(fmt, arg...) \
 printk(KERN_ERR "ENET: " fmt "\n", ##arg)


#else /* ENET_DEBUG == 0 */


#define ENET_DEBUG_DECLARE(x) /* */
#define PR_DEBUG(fmt, arg...) do { } while(0)


#endif /* ENET_DEBUG */


#if ENET_PROFILE

/* declare a variable for profiling */
#define ENET_PROFILING_DECLARE(x) x

/* get cpu timestamp ounter */
#define ENET_PROFILING_TIMESTAMP(x) \
 cpu_timestamp(x)

#define PR_PROFILING(fmt, arg...) \
 printk(KERN_ERR "ENET: " fmt "\n", ##arg)

#else /* !ENET_PROFILE */

#define ENET_PROFILING_DECLARE(x) /* */
#define ENET_PROFILING_TIMESTAMP(x) do { } while(0)
#define PR_PROFILING(fmt, arg...) do { } while(0)

#endif /* ENET_PROFILE */


#endif /*ENET_MODULE_H_*/
