/*
 * Copyright (c) 2006 The Trustees of Indiana University and Indiana
 *                    University Research and Technology
 *                    Corporation.  All rights reserved.
 * Copyright (c) 2006 The Technical University of Chemnitz. All 
 *                    rights reserved.
 *
 * Author(s): (alphabetically)
 *            Sven Eckelmann
 *            Torsten Hoefler 
 *            Mirko Reinhard 
 *            Matthias Treydte 
 */

#ifndef ENET_SOCKET_H_
#define ENET_SOCKET_H_

#include <linux/net.h>  /* struct socket, struct proto_ops */
#include <net/sock.h>  /* struct sock, basic socket operations */
#include <linux/netdevice.h> /* struct net_device, struct packet_type */
#include <linux/version.h> /* kernel version dependency */
#include "enet_compat.h"


/* wake up waiting threads to report socket errors */
#define SK_ERR_REPORT(__sk, __err)  \
 __sk->sk_err = -__err;   \
 if (!sock_flag(__sk, SOCK_DEAD)) \
  __sk->sk_error_report(__sk);


/*
 * ENET Ports
 */
static inline int sk_bound(struct sock *sk)
{
	return sk->sk_bind_node.pprev != NULL;
}


static inline int sk_unbound(struct sock *sk)
{
	return hlist_unhashed(&sk->sk_bind_node);
}


static inline void sk_del_bind_node_init(struct sock *sk)
{
	__sk_del_bind_node(sk);
	/* TODO: macht das Sinn? */
	sk_node_init(&sk->sk_bind_node);
}


/**
 * HTABLE_SK_FOR_EACH_BOUND - iterate over all sockets in a hashtable
 * @tpos: type* loop counter
 * @pos: &struct hlist_node loop counter
 * @hbucket: int hashtable bucket loop counter
 * @htable: hashtable
 * @hsize: int hash table size
 */
#define HTABLE_SK_FOR_EACH_BOUND(tpos, pos, hbucket, htable, hsize) \
 for (hbucket = 0; hbucket < hsize; ++hbucket)  \
  hlist_for_each_entry(tpos, pos, &htable[hbucket], sk_bind_node)


/*
 * PF_ENET socket operation prototypes
 */
int sock_enet_create(NET_PARAM struct socket *sock, int protocol);


#endif /*ENET_SOCKET_H_*/
